#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
请求类
"""
import logging
import uuid

from gep.api.response.GepResponse import GepResponse
from gep.api.util.ConfigUtil import *
from gep.api.util.HttpUtil import do_multipart_post, do_post
from gep.api.util.RSAUtil import RSAUtil

logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)s %(levelname)s %(message)s',
        filemode='a',)
logger = logging.getLogger('')

class ApiRequest(object):
    def __init__(self, charset="utf-8"):
        self.__charset = charset

    """
    内部方法，构造form表单输出结果
    """
    def __build_form(self, url, params):
        form = "<form name=\"punchout_form\" method=\"post\" action=\""
        form += url
        form += "\">\n"
        if params:
            for k, v in params.items():
                if not v:
                    continue
                form += "<input type=\"hidden\" name=\""
                form += k
                form += "\" value=\""
                form += v.replace("\"", "&quot;")
                form += "\">\n"
        form += "<input type=\"submit\" value=\"提交\" style=\"display:none\" >\n"
        form += "</form>\n"
        form += "<script>document.forms[0].submit();</script>"
        return form

    """
    内部方法，通过请求request对象构造请求查询字符串和业务参数
    """
    def __prepare_request(self, data_params):
        logger.info("[" + THREAD_LOCAL.uuid + "]data_content: %s" % data_params)
        data_content_str = RSAUtil.pri_encrypt(str(data_params), PFX_PATH, PFX_PSW)
        params = dict()
        params["version"] = VERSION
        params["certificateId"] = CERTIFICATE_ID
        params["agentNo"] = AGENT_NO
        params["dataType"] = DATA_TYPE
        params["apiType"] = API_TYPE
        params["dataContent"] = data_content_str
        if 'userNo' in data_params:
            params["userNo"] = data_params['userNo']

        return params

    """
    内部方法，解析请求返回结果并做验签
    """
    def __parse_response(self, response_str, skip_decrypt=False):
        if PYTHON_VERSION_3:
            response_str = response_str.decode(self.__charset)
        logger.info("[" + THREAD_LOCAL.uuid + "]response: %s" % response_str)

        response = GepResponse().parse(response_str)
        if response["success"] and not skip_decrypt:
            result = RSAUtil.pub_decrypt(response["result"], CER_PATH)
            logger.info("[" + THREAD_LOCAL.uuid + "]result decrypt: %s" % result)
            response["result"] = result

        return response_str

    """
    执行接口请求
    """
    def execute(self, path, data_params, multipart_params=None, skip_decrypt=False):
        THREAD_LOCAL.uuid = str(uuid.uuid1())

        headers = {
            'Content-type': 'application/x-www-form-urlencoded;charset=' + self.__charset,
            "Cache-Control": "no-cache",
            "Connection": "Keep-Alive",
        }

        params = self.__prepare_request(data_params)
        logger.info("[" + THREAD_LOCAL.uuid + "]request: %s" % params)
        url = SERVER_URL+path

        if multipart_params and len(multipart_params) > 0:
            response = do_multipart_post(url, None, headers, params, multipart_params,
                                         self.__charset, TIME_OUT)
        else:
            response = do_post(url, None, headers, params, self.__charset, TIME_OUT)

        return self.__parse_response(response, skip_decrypt)